@@ -3,8 +3,7 @@
  * \brief string tokeniser
  * \see mlt_tokeniser_s
  *
- * Copyright (C) 2002-2009 Ushodaya Enterprises Limited
- * \author Charles Yates <charles.yates@pandora.be>
+ * Copyright (C) 2002-2014 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +20,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef _MLT_TOKENISER_H_
-#define _MLT_TOKENISER_H_
+#ifndef MLT_TOKENISER_H
+#define MLT_TOKENISER_H
 
 /** \brief Tokeniser class
  *
