@@ -92,7 +92,7 @@ static int filter_get_audio( mlt_frame frame, void **buffer, mlt_audio_format *f
 				level = 41.0/42.0;
 		}
 		// max amplitude = 40/42, 3to10  oversamples=41, more then 10 oversamples=42
-		if ( level == 0.0 )
+		if ( level == 0.0 && num_samples > 0 )
 			level = val / num_samples * 40.0/42.0 / 127.0;
 		if ( iec_scale )
 			level = IEC_Scale( AMPTODBFS( level ) );
