@@ -1,7 +1,6 @@
 /*
  * consumer_xml.c -- a libxml2 serialiser of mlt service networks
- * Copyright (C) 2003-2014 Ushodaya Enterprises Limited
- * Author: Dan Dennedy <dan@dennedy.org>
+ * Copyright (C) 2003-2015 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -59,7 +58,7 @@ typedef struct serialise_context_s* serialise_context;
 
 static int consumer_start( mlt_consumer parent );
 static int consumer_stop( mlt_consumer parent );
-static int consumer_is_stopped( mlt_consumer this );
+static int consumer_is_stopped( mlt_consumer consumer );
 static void *consumer_thread( void *arg );
 static void serialise_service( serialise_context context, mlt_service service, xmlNode *node );
 
@@ -186,27 +185,27 @@ static char *xml_get_id( serialise_context context, mlt_service service, xml_typ
 mlt_consumer consumer_xml_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg )
 {
 	// Create the consumer object
-	mlt_consumer this = calloc( 1, sizeof( struct mlt_consumer_s ) );
+	mlt_consumer consumer = calloc( 1, sizeof( struct mlt_consumer_s ) );
 
 	// If no malloc'd and consumer init ok
-	if ( this != NULL && mlt_consumer_init( this, NULL, profile ) == 0 )
+	if ( consumer != NULL && mlt_consumer_init( consumer, NULL, profile ) == 0 )
 	{
 		// Allow thread to be started/stopped
-		this->start = consumer_start;
-		this->stop = consumer_stop;
-		this->is_stopped = consumer_is_stopped;
+		consumer->start = consumer_start;
+		consumer->stop = consumer_stop;
+		consumer->is_stopped = consumer_is_stopped;
 
-		mlt_properties_set( MLT_CONSUMER_PROPERTIES( this ), "resource", arg );
-		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( this ), "real_time", -1 );
-		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( this ), "prefill", 1 );
-		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( this ), "terminate_on_pause", 1 );
+		mlt_properties_set( MLT_CONSUMER_PROPERTIES( consumer ), "resource", arg );
+		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( consumer ), "real_time", 0 );
+		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( consumer ), "prefill", 1 );
+		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( consumer ), "terminate_on_pause", 1 );
 
 		// Return the consumer produced
-		return this;
+		return consumer;
 	}
 
 	// malloc or consumer init failed
-	free( this );
+	free( consumer );
 
 	// Indicate failure
 	return NULL;
@@ -725,7 +724,10 @@ xmlDocPtr xml_make_doc( mlt_consumer consumer, mlt_service service )
 	xmlDocSetRootElement( doc, root );
 
 	// Indicate the numeric locale
-	xmlNewProp( root, _x("LC_NUMERIC"), _x( setlocale( LC_NUMERIC, NULL ) ) );
+	if ( mlt_properties_get_lcnumeric( properties ) )
+		xmlNewProp( root, _x("LC_NUMERIC"), _x( mlt_properties_get_lcnumeric( properties ) ) );
+	else
+		xmlNewProp( root, _x("LC_NUMERIC"), _x( setlocale( LC_NUMERIC, NULL ) ) );
 
 	// Indicate the version
 	xmlNewProp( root, _x("version"), _x( mlt_version_get_string() ) );
@@ -733,7 +735,8 @@ xmlDocPtr xml_make_doc( mlt_consumer consumer, mlt_service service )
 	// If we have root, then deal with it now
 	if ( mlt_properties_get( properties, "root" ) != NULL )
 	{
-		xmlNewProp( root, _x("root"), _x(mlt_properties_get( properties, "root" )) );
+		if ( !mlt_properties_get_int( MLT_CONSUMER_PROPERTIES( consumer ), "no_root" ) )
+			xmlNewProp( root, _x("root"), _x(mlt_properties_get( properties, "root" )) );
 		context->root = strdup( mlt_properties_get( properties, "root" ) );
 	}
 	else
@@ -746,8 +749,10 @@ xmlDocPtr xml_make_doc( mlt_consumer consumer, mlt_service service )
 	context->no_meta = mlt_properties_get_int( MLT_CONSUMER_PROPERTIES( consumer ), "no_meta" );
 	const char *time_format = mlt_properties_get( MLT_CONSUMER_PROPERTIES( consumer ), "time_format" );
 	if ( time_format && ( !strcmp( time_format, "smpte" ) || !strcmp( time_format, "SMPTE" )
-			|| !strcmp( time_format, "timecode" ) ) )
-		context->time_format = mlt_time_smpte;
+			|| !strcmp( time_format, "timecode" ) || !strcmp( time_format, "smpte_df" ) ) )
+		context->time_format = mlt_time_smpte_df;
+	else if ( time_format && ( !strcmp( time_format, "smpte_ndf" ) ) )
+		context->time_format = mlt_time_smpte_ndf;
 	else if ( time_format && ( !strcmp( time_format, "clock" ) || !strcmp( time_format, "CLOCK" ) ) )
 		context->time_format = mlt_time_clock;
 
@@ -813,11 +818,11 @@ xmlDocPtr xml_make_doc( mlt_consumer consumer, mlt_service service )
 }
 
 
-static void output_xml( mlt_consumer this )
+static void output_xml( mlt_consumer consumer )
 {
 	// Get the producer service
-	mlt_service service = mlt_service_producer( MLT_CONSUMER_SERVICE( this ) );
-	mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+	mlt_service service = mlt_service_producer( MLT_CONSUMER_SERVICE( consumer ) );
+	mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
 	char *resource =  mlt_properties_get( properties, "resource" );
 	xmlDocPtr doc = NULL;
 
@@ -842,14 +847,8 @@ static void output_xml( mlt_consumer this )
 		free( cwd );
 	}
 
-#if !defined(__GLIBC__) && !defined(__DARWIN__)
-	// Get the current locale
-	char *orig_localename = strdup( setlocale( LC_NUMERIC, NULL ) );
-	setlocale( LC_NUMERIC, "C" );
-#endif
-
 	// Make the document
-	doc = xml_make_doc( this, service );
+	doc = xml_make_doc( consumer, service );
 
 	// Handle the output
 	if ( resource == NULL || !strcmp( resource, "" ) )
@@ -877,18 +876,12 @@ static void output_xml( mlt_consumer this )
 		xmlSaveFormatFileEnc( resource, doc, "utf-8", 1 );
 	}
 
-#if !defined(__GLIBC__) && !defined(__DARWIN__)
-	// Restore the current locale
-	setlocale( LC_NUMERIC, orig_localename );
-	free( orig_localename );
-#endif
-
 	// Close the document
 	xmlFreeDoc( doc );
 }
-static int consumer_start( mlt_consumer this )
+static int consumer_start( mlt_consumer consumer )
 {
-	mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+	mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
 
 	if ( mlt_properties_get_int( properties, "all" ) )
 	{
@@ -906,28 +899,28 @@ static int consumer_start( mlt_consumer this )
 			mlt_properties_set_int( properties, "joined", 0 );
 
 			// Create the thread
-			pthread_create( thread, NULL, consumer_thread, this );
+			pthread_create( thread, NULL, consumer_thread, consumer );
 		}
 	}
 	else
 	{
-		output_xml( this );
-		mlt_consumer_stop( this );
-		mlt_consumer_stopped( this );
+		output_xml( consumer );
+		mlt_consumer_stop( consumer );
+		mlt_consumer_stopped( consumer );
 	}
 	return 0;
 }
 
-static int consumer_is_stopped( mlt_consumer this )
+static int consumer_is_stopped( mlt_consumer consumer )
 {
-	mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+	mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
 	return !mlt_properties_get_int( properties, "running" );
 }
 
-static int consumer_stop( mlt_consumer this )
+static int consumer_stop( mlt_consumer consumer )
 {
 	// Get the properties
-	mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+	mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
 
 	// Check that we're running
 	if ( !mlt_properties_get_int( properties, "joined" ) )
@@ -950,10 +943,10 @@ static int consumer_stop( mlt_consumer this )
 static void *consumer_thread( void *arg )
 {
 	// Map the argument to the object
-	mlt_consumer this = arg;
+	mlt_consumer consumer = arg;
 
 	// Get the properties
-	mlt_properties properties = MLT_CONSUMER_PROPERTIES( this );
+	mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
 
 	// Convenience functionality
 	int terminate_on_pause = mlt_properties_get_int( properties, "terminate_on_pause" );
@@ -969,7 +962,7 @@ static void *consumer_thread( void *arg )
 	while( !terminated && mlt_properties_get_int( properties, "running" ) )
 	{
 		// Get the frame
-		frame = mlt_consumer_rt_frame( this );
+		frame = mlt_consumer_rt_frame( consumer );
 
 		// Check for termination
 		if ( terminate_on_pause && frame != NULL )
@@ -996,11 +989,11 @@ static void *consumer_thread( void *arg )
 			mlt_frame_close( frame );
 		}
 	}
-	output_xml( this );
+	output_xml( consumer );
 
 	// Indicate that the consumer is stopped
 	mlt_properties_set_int( properties, "running", 0 );
-	mlt_consumer_stopped( this );
+	mlt_consumer_stopped( consumer );
 
 	return NULL;
 }
